#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/sleep.h>
#include "photoresistor.h"

unsigned int random_seed = 0;

// check to see if it's dark out
int is_dark()
{
  unsigned int average = 0, i;

  // enable interrupts
  sei();

  // setup ADC
  ADMUX = _BV(ADLAR) | _BV(MUX1); // ADLAR = 8-bit resolution, MUX1 = PB4/ADC2
  ADCSRA = _BV(ADEN) | _BV(ADIE); // ADEN = enable ADC, ADIE = interrupt mode
  set_sleep_mode(SLEEP_MODE_ADC); // Set sleep mode ADC to get a better reading

  // read photo resistor average
  for(i = 0; i < PR_AVERAGES; i++)
  {
    sleep_mode();
    average += ADCH;
  }

  // set random seed using an actual random source!!!
  random_seed += average;

  // turn off ADC
  ADCSRA = 0;

  // is it dark?
  return (average / PR_AVERAGES) <= PR_THRESHOLD;
}

// sleep the CPU for 8 seconds
void sleep_8s()
{
  // enable interrupts
  sei();

  // set sleep mode
  set_sleep_mode(SLEEP_MODE_PWR_DOWN);

  // start watchdog timer
  WDTCR = _BV(WDIE) | _BV(WDP3) | _BV(WDP0);

  // sleep for 8 seconds
  sleep_mode();

  // disable watchdog timer
  WDTCR = 0;
}

// return the random seed
unsigned int get_random_seed()
{
  return random_seed;
}

// these interrupt are just used to wake the CPU
ISR(ADC_vect)
{
}

ISR(WDT_vect)
{
}
